from SeqADT import SeqT as SeqT
from CurveADT import CurveT as CurveT

def assertionEqual(test, result, name):
  if test == result:
    print("Test passed, %s == %s, %s " % (test, result, name))
  else:
    print("Test failed, %s != %s, %s " % (test, result, name))


def assertionApproximatelyEqual(test, result, error, name):
  if abs(test - result) < error:
    print("Test passed, Actual: %s  Approximate: %s, %s " % (test, result, name))
  else:
    print("Test failed, Actual: %s  Approximate: %s, %s " % (test, result, name))

def testSeq1():
  seq = SeqT()
  # A constructor (SeqT()) that takes no arguments and creates an object
  # whose state consists of an empty sequence
  assertionEqual(seq.size(), 0, "Empty sequence")

def testSeq2():
  seq = SeqT()
  # add immediately after the last entry in the existing sequence.
  seq.add(0, 2.0)
  assertionEqual(seq.seq[0], 2.0, "Add to an empty sequence")

def testSeq3():
  seq = SeqT()
  # add immediately after the last entry in the existing sequence.
  seq.add(52, 2.0)
  assertionEqual(seq.seq[0], 2.0, "Add to an empty sequence large index")

def testSeq4():
  seq = SeqT()
  # add immediately after the last entry in the existing sequence.
  seq.add(1, 2.0)
  seq.add(1, 3.0)
  seq.add(1, 4.0)
  seq.add(1, 5.0)
  # Values can only be added within the existing sequence
  assertionEqual(seq.seq[1], 5.0, "Add within sequence")

def testSeq5():
  seq = SeqT()
  seq.add(0, 1)
  seq.add(1, 2)
  seq.add(2, 3)
  seq.add(3, 4)
  seq.rm(0)
  #A call to this method modifies the sequence so that the entry at index i
  # is removed. The length of the list will decrease by 1.
  assertionEqual(seq.seq[0], 2, "removed 0th element")
  assertionEqual(seq.size(), 3, "size decreased by 1")

def testSeq6():
  seq = SeqT()
  # undefined behaviour
  seq.rm(0)

def testSeq7():
  seq = SeqT()
  # undefined behaviour
  seq.set(0, 1)

def testSeq8():
  seq = SeqT()
  seq.add(0, 9)
  seq.set(0, 1)
  assertionEqual(seq.seq[0], 1, "mutated item at index 0")

def testSeq9():
  seq = SeqT()
  # undefined behaviour
  seq.get(0)

def testSeq10():
  seq = SeqT()
  seq.add(0, 1)
  seq.add(1, 2)
  seq.add(2, 3)
  assertionEqual(seq.get(2), 3, "got correct item")


def testSeq11():
  seq = SeqT()
  assertionEqual(seq.size(), 0, "Empty sequence, size 0")

def testSeq12():
  seq = SeqT()
  seq.add(0, 1)
  seq.add(1, 2)
  seq.add(2, 3)
  assertionEqual(seq.size(), 3, "Non empty sequence, correct size")

def testSeq13():
  seq = SeqT()
  seq.add(0, 1.0)
  seq.add(3, 2.0)
  seq.add(4, 5.0)
  seq.rm(2)
  assertionEqual(seq.size(), 2, "Size correct after removal")


def testSeq14():
  seq = SeqT()
  seq.add(0, 1.0)
  seq.add(3, 2.0)
  seq.add(4, 5.0)
  assertionEqual(seq.indexInSeq(2.0), 0, "Correct index")

def testSeq14():
  seq = SeqT()
  seq.add(0, 1.0)
  seq.add(3, 2.0)
  seq.add(4, 5.0)
  assertionEqual(seq.indexInSeq(7.0), -1.0, "Correct index")

def testCurve1():
  curve = CurveT("./src/linear")
  assertionApproximatelyEqual(4.85, curve.linVal(2.5), 0.000005, "Correct approximation")

def testCurve2():
  curve = CurveT("./src/quad")
  assertionApproximatelyEqual(31.575, curve.quadVal(2.5), 0.000005, "Correct approximation")

def testCurve3():
  curve = CurveT("./src/points")
  try:
    curve.linVal(0)
    print("Test failed: No exception! divion by zero!")
  except Exception as err:
    if type(err) == ZeroDivisionError:
      print("Test passed: Exception: {0}".format(err))
    else:
      print("Test failed: Wrong Exception: {0} should be ZeroDivisionError".format(err))

def testCurve4():
    curve = CurveT("./src/point")
    try:
      curve.linVal(12)
      print("Test failed: No exception! not enough points for quadratic interpolation")
    except Exception as err:
      if type(err) == ValueError:
        print("Test passed: {0}".format(err))
      else:
        print("Test failed: Wrong Exception: {0} should be {1}".format(err, ValueError))


def testCurve5():
  curve = CurveT("./src/points")
  try:
    curve.quadVal(12)
    print("Test failed: No exception! not enough points for quadratic interpolation")
  except Exception as err:
    if type(err) == ValueError:
      print("Test passed: {0}".format(err))
    else:
      print("Test failed: Wrong Exception: {0} should be {1}".format(err, ValueError))

def testCurve6():
  curve = CurveT("./src/vertical")
  try:
    curve.quadVal(12)
    print("Test failed: No exception! division by zero!")
  except Exception as err:
    if type(err) == ZeroDivisionError:
      print("Test passed: {0}".format(err))
    else:
      print("Test failed: Wrong Exception: {0} should be {1}".format(err, ZeroDivisionError))

def testCurve6():
  curve = CurveT("./src/vertical")
  try:
    curve.linVal(12)
    print("Test failed: No exception! division by zero!")
  except Exception as err:
    if type(err) == ZeroDivisionError:
      print("Test passed: {0}".format(err))
    else:
      print("Test failed: Wrong Exception: {0} should be {1}".format(err, ZeroDivisionError))


def test():
  testSeq1()
  testSeq2()
  testSeq3()
  testSeq4()
  testSeq5()
  #testSeq6() undefined behaviour
  #testSeq7() undefined behaviour
  testSeq8()
  #testSeq9() undefined behaviour
  testSeq10()
  testSeq11()
  testSeq12()
  testSeq13()
  testSeq14()
  testCurve1()
  testCurve2()
  testCurve3()
  testCurve4()
  testCurve5()
  testCurve6()


test()
