## @file Plot.py
#  @author Steven Palmer and Henry Madej
#  @brief Plotting functions
#  @date 20/02/2018

from matplotlib.pyplot import *
from Exceptions import *
from CurveADT import *

## @brief PlotSeq plots a sequence of (x, y) pairs
#  @param X corresponding x values of the sequence
#  @param Y corresponding y values of the sequence
def PlotSeq(X, Y):
  if len(X) != len(Y):
    raise SeqSizeMismatch
    
  figure(dpi=100)
  plot(X, Y)

  show()
  

## @brief PlotCurve plots a curve
#  @param c the curve to be plotted
#  @param n number of equally spaced points
def PlotCurve(c, n):
  d = (c.maxD() - c.minD())/(n+1)
  
  curX = c.minD()
  
  X = []
  Y = []
    
  for i in range(n):
    curX = curX + d
    X.append(curX)
    Y.append(c.eval(curX))
    
  PlotSeq(X, Y)
